#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import os
import sys

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")
sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

import pytest
from libs.exceptions import FileSourceError, GCSObjectURIError
from libs.source_file_check import GCSSourceFileChecker


class TestSourceFileChecker:

    @pytest.fixture()
    def file_checker(self):
        return GCSSourceFileChecker()

    @pytest.fixture()
    def mock_file_exist_under_uri(self, monkeypatch, file_exists: bool):
        """
        Mock response from GCS if file exists or not.
        """
        monkeypatch.setattr(GCSSourceFileChecker, "_does_file_exist_in_bucket",
                            lambda *args, **kwargs: file_exists)

    @pytest.mark.parametrize(
        "file_path, file_exists",
        [
            pytest.param("gs://test/test", True, id="Valid URI")
        ]
    )
    def test_does_file_exist_in_bucket(
        self,
        monkeypatch,
        mock_file_exist_under_uri,
        file_exists: bool,
        file_checker: GCSSourceFileChecker,
        file_path: str
    ):
        """
        Test if file does really exist.
        """
        file_checker.does_file_exist(file_path)

    @pytest.mark.parametrize(
        "file_path, file_exists",
        [
            pytest.param("gs://test/test", False)
        ]
    )
    def test_file_does_not_exist_in_bucket(
        self,
        monkeypatch,
        mock_file_exist_under_uri,
        file_exists: bool,
        file_checker: GCSSourceFileChecker,
        file_path: str
    ):
        """
        Test if file doesn't exist.
        """
        with pytest.raises(FileSourceError):
            file_checker.does_file_exist(file_path)

    @pytest.mark.parametrize(
        "file_path",
        [
            pytest.param("gs://test"),
            pytest.param("://test"),
            pytest.param("test"),
        ]
    )
    def test_invalid_gcs_object_uri(self, file_checker: GCSSourceFileChecker, file_path):
        with pytest.raises(GCSObjectURIError):
            file_checker._parse_object_uri(file_path)
