#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import http
import json
import os
import sys

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")
sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

from libs.context import Context
from libs.upload_file import GCSFileUploader
from libs.refresh_token import AirflowTokenRefresher
from libs.exceptions import EmptyManifestError
from deepdiff import DeepDiff
import pytest
import requests

from file_paths import (
    MANIFEST_WELLBORE_VALID_PATH,
    MANIFEST_EMPTY_PATH,
    MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH,
    RECORD_WELLBORE_VALID_PATH,
    RECORD_SEISMIC_TRACE_DATA_VALID_PATH,
)
from libs import process_manifest_r3

TENANT = "opendes"


class TestManifestProcessor:

    @staticmethod
    def monkeypatch_storage_response(
        monkeypatch,
        response_content: bytes = b"{\"recordIds\": [\"test\"]}"
    ):
        """
        Make storage service request return mock response.
        """

        def mockresponse(url, data=None, **kwargs):
            response = requests.Response()
            response.status_code = http.HTTPStatus.OK
            response._content = response_content
            return response

        monkeypatch.setattr(requests, "put", mockresponse)

    @staticmethod
    def monkeypatch_storage_response_error(monkeypatch, error_status: http.HTTPStatus):
        """
        Make storage request return HTTPError response.
        """

        def mockresponse(url, data=None, **kwargs):
            response = requests.Response()
            response.status_code = error_status
            response._content = b"{\"recordIds\": [\"test\"]}"
            return response

        monkeypatch.setattr(requests, "put", mockresponse)

    @pytest.fixture(autouse=True)
    def manifest_processor(self, monkeypatch, conf_path: str):
        with open(conf_path) as f:
            conf = json.load(f)
        context = Context.populate(conf)
        token_refresher = AirflowTokenRefresher()
        file_uploader = GCSFileUploader("test", token_refresher, context)
        manifest_processor = process_manifest_r3.ManifestProcessor(
            storage_url="",
            dagrun_conf=conf,
            token_refresher=token_refresher,
            context=context,
            file_uploader = file_uploader
        )

        monkeypatch.setattr(manifest_processor, "generate_id", lambda manifest: "test_id")
        monkeypatch.setattr(file_uploader, "upload_file",
                            lambda *args, **kwargs: "test")
        return manifest_processor

    @pytest.fixture()
    def mock_records_list(self, records_file_path: str):
        """
        Mock records list from Storage service taken from file.
        """
        with open(records_file_path) as f:
            return json.load(f)

    @pytest.mark.parametrize(
        "conf_path,records_file_path",
        [
            pytest.param(MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH,
                         RECORD_SEISMIC_TRACE_DATA_VALID_PATH,
                         id="Valid WorkProduct"),
            pytest.param(MANIFEST_WELLBORE_VALID_PATH,
                         RECORD_WELLBORE_VALID_PATH,
                         id="Valid Wellbore"),
        ]
    )
    def test_create_manifest_records(
        self,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        mock_records_list: list,
        conf_path: str,
        records_file_path: str
    ):
        actual_records = manifest_processor.create_manifest_records()
        sort_predicate = lambda x: x["kind"]
        actual_records.sort(key=sort_predicate)
        mock_records_list.sort(key=sort_predicate)
        assert actual_records == mock_records_list, DeepDiff(actual_records, mock_records_list,
                                                             ignore_order=True)

    @pytest.mark.parametrize(
        "conf_path,records_file_path",
        [
            pytest.param(
                MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH,
                RECORD_SEISMIC_TRACE_DATA_VALID_PATH,
                id="Valid WorkProduct"
            ),
        ]
    )
    def test_save_record(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        mock_records_list: list,
        conf_path: str,
        records_file_path: str
    ):
        self.monkeypatch_storage_response(monkeypatch)
        manifest_processor.save_record({}, records_file_path)

    @pytest.mark.parametrize(
        "conf_path",
        [
            MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH
        ]
    )
    def test_save_record_invalid_storage_response_value(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        conf_path: str
    ):
        self.monkeypatch_storage_response(monkeypatch, b"{}")
        with pytest.raises(ValueError):
            manifest_processor.save_record({}, [{}])

    @pytest.mark.parametrize(
        "conf_path",
        [
            pytest.param(MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH, id="Valid WorkProduct")
        ]
    )
    def test_save_record_storage_response_http_error(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        conf_path: str
    ):
        self.monkeypatch_storage_response_error(monkeypatch, http.HTTPStatus.INTERNAL_SERVER_ERROR)
        with pytest.raises(requests.HTTPError):
            manifest_processor.save_record({}, conf_path)

    @pytest.mark.parametrize(
        "conf_path",
        [
            pytest.param(MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH, id="WorkProduct"),
            pytest.param(MANIFEST_WELLBORE_VALID_PATH, id="Master"),

        ]
    )
    def test_process_manifest_valid(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        conf_path: str
    ):
        self.monkeypatch_storage_response(monkeypatch)
        manifest_processor.process_manifest()

    @pytest.mark.parametrize(
        "conf_path",
        [
            pytest.param(MANIFEST_EMPTY_PATH, id="Empty Manifest"),
        ]
    )
    def test_process_empty_manifest(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        conf_path: str
    ):
        self.monkeypatch_storage_response(monkeypatch)
        with pytest.raises(EmptyManifestError):
            manifest_processor.process_manifest()

    @pytest.mark.parametrize(
        "conf_path,expected_kind_name",
        [
            pytest.param(MANIFEST_WELLBORE_VALID_PATH, "Wellbore", id="Valid Wellbore"),
        ]
    )
    def test_get_kind(
        self,
        monkeypatch,
        manifest_processor: process_manifest_r3.ManifestProcessor,
        conf_path: str,
        expected_kind_name: str
    ):
        for manifest_part in manifest_processor.data_object["manifest"]:
            kind = manifest_part["kind"]
            assert expected_kind_name == manifest_processor._get_kind_name(kind)

    @pytest.mark.parametrize(
        "conf_path,expected_kind_name",
        [
            pytest.param(MANIFEST_WELLBORE_VALID_PATH, "Wellbore", id="Valid Wellbore"),
        ]
    )
    def test_generate_id(
        self,
        monkeypatch,
        conf_path: str,
        expected_kind_name: str
    ):
        with open(conf_path) as f:
            conf = json.load(f)
        context = process_manifest_r3.Context.populate(conf)
        manifest_processor = process_manifest_r3.ManifestProcessor(
            storage_url="",
            dagrun_conf=conf,
            token_refresher=AirflowTokenRefresher(),
            context=context,
            file_uploader=GCSFileUploader("test", AirflowTokenRefresher(), context)
        )
        for manifest_part in manifest_processor.data_object["manifest"]:
            group_type = manifest_part["groupType"]
            kind_name = manifest_processor._get_kind_name(manifest_part["kind"])
            generated_id = manifest_processor.generate_id(manifest_part)
            assert generated_id.startswith(f"{TENANT}:{group_type}_{kind_name}:")
