#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import json
import os
import sys

import pytest
from google.oauth2 import service_account

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

from libs.refresh_token import AirflowTokenRefresher
from libs.exceptions import SAFilePathError


class MockCredentials:

    def __init__(self, access_token):
        self.access_token = access_token

    def refresh(self, *args, **kwargs):
        self.token = self.access_token


class TestAirflowTokenRefresher:

    @pytest.fixture()
    def token_refresher(self) -> AirflowTokenRefresher:
        token_refresher = AirflowTokenRefresher()
        return token_refresher

    @pytest.fixture()
    def mock_sa_file_info(self, monkeypatch, token_refresher: AirflowTokenRefresher,
                          sa_file_path: str):
        """
        Return fake sa_file_info from fake path in environment.
        """
        monkeypatch.setattr(os.environ, "get", lambda *args, **kwargs: sa_file_path)
        monkeypatch.setattr(json, "load", lambda *args, **kwargs: "test")
        monkeypatch.setattr(os.path, "isfile", lambda *args, **kwargs: True)
        monkeypatch.setattr(token_refresher, "get_sa_info_from_file",
                            lambda *args, **kwargs: "test")
        monkeypatch.setattr(token_refresher, "get_sa_info_from_google_storage",
                            lambda *args, **kwargs: "test")

    @pytest.fixture()
    def mock_sa_credentials(self, monkeypatch, expected_token: str):
        """
        Return fake service account credentials having expected token.
        """
        monkeypatch.setattr(service_account.Credentials,
                            "from_service_account_info",
                            lambda *args, **kwargs: MockCredentials(expected_token))

    @pytest.mark.parametrize(
        "access_token",
        [
            "test"
        ]
    )
    def test_access_token_cached(self, monkeypatch, token_refresher: AirflowTokenRefresher,
                                 access_token: str):
        """
        Check if access token stored in Airflow Variables after refreshing it.
        """
        monkeypatch.setattr(token_refresher, "get_access_token_using_sa_file",
                            lambda *args, **kwargs: access_token)
        token_refresher.refresh_token()
        assert token_refresher.airflow_variables.get("access_token") == access_token

    @pytest.mark.parametrize(
        "access_token",
        [
            "test",
            "aaaa"
        ]
    )
    def test_authorization_header(self, monkeypatch, token_refresher: AirflowTokenRefresher,
                                  access_token: str):
        """
        Check if Authorization header is 'Bearer <access_token>'
        """
        monkeypatch.setattr(token_refresher, "get_access_token_using_sa_file",
                            lambda *args, **kwargs: access_token)
        token_refresher.refresh_token()
        assert token_refresher.authorization_header.get("Authorization") == f"Bearer {access_token}"

    def test_raise_sa_path_error_on_getting_absent_sa_file(self,
                                                           token_refresher: AirflowTokenRefresher):
        """
        Check if error raises if sa file path is empty
        """
        with pytest.raises(SAFilePathError):
            token_refresher.get_sa_info()

    @pytest.mark.parametrize(
        "expected_token, sa_file_path",
        [
            pytest.param("test1", "/test", id="Local file"),
            pytest.param("test2", "gs://test/test", id="GCS")
        ]
    )
    def test_refresh_token_using_sa_file(
        self,
        monkeypatch,
        mock_sa_file_info,
        mock_sa_credentials,
        token_refresher: AirflowTokenRefresher,
        sa_file_path: str,
        expected_token: str,
    ):
        token_refresher.refresh_token()
        assert token_refresher.access_token == expected_token

    @pytest.mark.parametrize(
        "expected_token, sa_file_path",
        [
            pytest.param("test1", "/test", id="Local file"),
            pytest.param("test2", "gs://test/test", id="GCS")
        ]
    )
    def test_refresh_token_using_sa_file_no_cached_variable(
        self,
        monkeypatch,
        mock_sa_file_info,
        mock_sa_credentials,
        token_refresher: AirflowTokenRefresher,
        sa_file_path: str,
        expected_token: str,
    ):
        """
        Check if token refreshes automatically if key is not stored in cache.
        """

        def mock_empty_variable(key):
            """
            Raise error as if Airflow Variable don't have 'access_token' variable.
            """
            raise KeyError

        monkeypatch.setattr(token_refresher.airflow_variables, "get", mock_empty_variable)
        access_token = token_refresher.access_token
        assert access_token == expected_token
