#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import copy
import http
import requests
import json
import os
import sys

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")
sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

from file_paths import (
    DATA_PATH_PREFIX,
    MANIFEST_EMPTY_PATH,
    SCHEMA_FILE_VALID_PATH,
    SCHEMA_SEISMIC_TRACE_DATA_VALID_PATH,
    SCHEMA_WORK_PRODUCT_VALID_PATH,
    MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH,
    MANIFEST_WELLBORE_VALID_PATH,
    SCHEMA_WELLBORE_VALID_PATH
)
from mock_responses import MockSchemaResponse
from libs.context import Context
from libs.refresh_token import AirflowTokenRefresher
from libs.exceptions import EmptyManifestError, NotOSDUShemaFormatError
import pytest

from libs.validate_schema import SchemaValidator

TENANT = "opendes"


class TestSchemaValidator:

    @pytest.fixture
    def schema_validator(self, monkeypatch, manifest_file: str, schema_file: str):
        with open(manifest_file) as f:
            manifest_file = json.load(f)
        conf = copy.deepcopy(manifest_file)
        context = Context.populate(conf)
        validator = SchemaValidator(
            "",
            conf,
            AirflowTokenRefresher(),
            context
        )
        if schema_file:
            monkeypatch.setattr(requests, "get",
                                lambda *args, **kwargs: MockSchemaResponse(schema_file))
        return validator

    @pytest.mark.parametrize(
        "manifest_file,schema_file",
        [
            pytest.param(
                MANIFEST_SEISMIC_TRACE_DATA_VALID_PATH, None,
                id="Valid manifest_file")
        ]
    )
    def test_schema_validator_validate_workproduct_manifest(self,
                                                            monkeypatch,
                                                            schema_validator: SchemaValidator,
                                                            manifest_file: str,
                                                            schema_file: str):
        def mock_get_schema(uri: str):
            if ":WorkProduct:" in uri:
                schema_path = SCHEMA_WORK_PRODUCT_VALID_PATH
            elif ":SeismicTraceData:" in uri:
                schema_path = SCHEMA_SEISMIC_TRACE_DATA_VALID_PATH
            elif ":File:" in uri:
                schema_path = SCHEMA_FILE_VALID_PATH
            else:
                print(uri)
                raise
            with open(schema_path) as f:
                schema_content = json.load(f)
            return schema_content

        monkeypatch.setattr(schema_validator, "get_schema_request", mock_get_schema)
        schema_validator.validate_manifest()

    @pytest.mark.parametrize(
        "manifest_file,schema_file",
        [
            pytest.param(
                MANIFEST_WELLBORE_VALID_PATH,
                SCHEMA_WELLBORE_VALID_PATH,
                id="Valid manifest Wellore"),
        ]
    )
    def test_schema_validator_master_manifest(self, monkeypatch, schema_validator, manifest_file,
                                              schema_file):
        def mock_get_schema(uri: str):
            with open(schema_file) as f:
                schema_content = json.load(f)
            return schema_content

        monkeypatch.setattr(schema_validator, "get_schema_request", mock_get_schema)
        schema_validator.validate_manifest()

    @pytest.mark.parametrize(
        "manifest_file,schema_file",
        [
            pytest.param(
                MANIFEST_EMPTY_PATH, None,
                id="Empty Manifest"),
        ]
    )
    def test_schema_validator_empty_manifest(self,
                                             monkeypatch,
                                             schema_validator: SchemaValidator,
                                             manifest_file: str,
                                             schema_file: str):
        with pytest.raises(EmptyManifestError):
            schema_validator.validate_manifest()

    @pytest.mark.parametrize(
        "manifest_file,schema_file",
        [
            pytest.param(
                f"{DATA_PATH_PREFIX}/invalid/NotOsduFormat.json", None,
                id="Not OSDU FORMAT"),
        ]
    )
    def test_schema_validator_not_osdu_format(self,
                                              monkeypatch,
                                              schema_validator: SchemaValidator,
                                              manifest_file: str,
                                              schema_file: str):
        with pytest.raises(NotOSDUShemaFormatError):
            schema_validator.validate_manifest()

    @pytest.mark.parametrize(
        "manifest_file,schema_file,kind",
        [
            pytest.param(
                MANIFEST_WELLBORE_VALID_PATH,
                SCHEMA_WELLBORE_VALID_PATH,
                "opendes:osdu:Wellbore:0.3.0",
                id="Valid manifest Wellore"),
        ]
    )
    def test_get_schema_request(self,
                                schema_validator: SchemaValidator,
                                manifest_file: str,
                                schema_file: str,
                                kind: str):
        schema_validator.get_schema_request(kind)

    @pytest.mark.parametrize(
        "manifest_file,schema_file,kind",
        [
            pytest.param(
                MANIFEST_WELLBORE_VALID_PATH,
                SCHEMA_WELLBORE_VALID_PATH,
                "opendes:osdu:Wellbore:0.3.0",
                id="Valid manifest Wellore"),
        ]
    )
    def test_get_schema_error(self,
                              monkeypatch,
                              schema_validator: SchemaValidator,
                              manifest_file: str,
                              schema_file: str,
                              kind: str):
        monkeypatch.setattr(requests,
                            "get",
                            lambda *args, **kwargs: MockSchemaResponse("{}",
                                                                       http.HTTPStatus.INTERNAL_SERVER_ERROR))
        with pytest.raises(requests.HTTPError):
            schema_validator.get_schema(kind)
